# Copyright (c) 2019 The Regents of the University of Michigan
# All rights reserved.
# This software is licensed under the BSD 3-Clause License.
"""Define the signac-flow version."""

__version__ = "0.21.0"

__all__ = ["__version__"]
